import Letter from "./letter.js";

export class Parser {
    constructor(layout) {
        this._layout = layout;
    }

    _vorne0 = {
        3: [2, 3, 4],
        4: [1, 5],
        5: [1, 4, 5],
        6: [1, 3, 5],
        7: [1, 2, 5],
        8: [1, 5],
        9: [2, 3, 4]
    };
    _hinten0 = {
        3: [8, 9, 10],
        4: [7, 11],
        5: [7, 10, 11],
        6: [7, 9, 11],
        7: [7, 8, 11],
        8: [7, 11],
        9: [8, 9, 10]
    };
    _vorne1 = {
        3: [3],
        4: [2, 3],
        5: [3],
        6: [3],
        7: [3],
        8: [3],
        9: [2, 3, 4]
    };
    _hinten1 = {
        3: [9],
        4: [8, 9],
        5: [9],
        6: [9],
        7: [9],
        8: [9],
        9: [8, 9, 10]
    };
    _vorne2 = {
        3: [2, 3, 4],
        4: [1, 5],
        5: [5],
        6: [4],
        7: [3],
        8: [2],
        9: [1, 2, 3, 4, 5]
    };
    _hinten2 = {
        3: [8, 9, 10],
        4: [7, 11],
        5: [11],
        6: [10],
        7: [9],
        8: [8],
        9: [7, 8, 9, 10, 11]
    };
    _vorne3 = {
        3: [1, 2, 3, 4, 5],
        4: [4],
        5: [3],
        6: [4],
        7: [5],
        8: [1, 5],
        9: [2, 3, 4]
    };
    _hinten3 = {
        3: [7, 8, 9, 10, 11],
        4: [10],
        5: [9],
        6: [10],
        7: [11],
        8: [7, 11],
        9: [8, 9, 10]
    };
    _vorne4 = {
        3: [4],
        4: [3, 4],
        5: [2, 4],
        6: [1, 4],
        7: [1, 2, 3, 4, 5],
        8: [4],
        9: [4]
    };
    _hinten4 = {
        3: [10],
        4: [9, 10],
        5: [8, 10],
        6: [7, 10],
        7: [7, 8, 9, 10, 11],
        8: [10],
        9: [10]
    };
    _vorne5 = {
        3: [1, 2, 3, 4, 5],
        4: [1],
        5: [1, 2, 3, 4],
        6: [5],
        7: [5],
        8: [1, 5],
        9: [2, 3, 4]
    };
    _hinten5 = {
        3: [7, 8, 9, 10, 11],
        4: [7],
        5: [7, 8, 9, 10],
        6: [11],
        7: [11],
        8: [7, 11],
        9: [8, 9, 10]
    };
    _hinten6 = {
        3: [9, 10],
        4: [8],
        5: [7],
        6: [7, 8, 9, 10],
        7: [7, 11],
        8: [7, 11],
        9: [8, 9, 10]
    };
    _hinten7 = {
        3: [7, 8, 9, 10, 11],
        4: [11],
        5: [10],
        6: [9],
        7: [8],
        8: [8],
        9: [8]
    };
    _hinten8 = {
        3: [8, 9, 10],
        4: [7, 11],
        5: [7, 11],
        6: [8, 9, 10],
        7: [7, 11],
        8: [7, 11],
        9: [8, 9, 10]
    };
    _hinten9 = {
        3: [8, 9, 10],
        4: [7, 11],
        5: [7, 11],
        6: [8, 9, 10, 11],
        7: [11],
        8: [10],
        9: [8, 9]
    };
    _seconds = {
        "0": [this._vorne0, this._hinten0],
        "1": [this._vorne0, this._hinten1],
        "2": [this._vorne0, this._hinten2],
        "3": [this._vorne0, this._hinten3],
        "4": [this._vorne0, this._hinten4],
        "5": [this._vorne0, this._hinten5],
        "6": [this._vorne0, this._hinten6],
        "7": [this._vorne0, this._hinten7],
        "8": [this._vorne0, this._hinten8],
        "9": [this._vorne0, this._hinten9],
        "10": [this._vorne1, this._hinten0],
        "11": [this._vorne1, this._hinten1],
        "12": [this._vorne1, this._hinten2],
        "13": [this._vorne1, this._hinten3],
        "14": [this._vorne1, this._hinten4],
        "15": [this._vorne1, this._hinten5],
        "16": [this._vorne1, this._hinten6],
        "17": [this._vorne1, this._hinten7],
        "18": [this._vorne1, this._hinten8],
        "19": [this._vorne1, this._hinten9],
        "20": [this._vorne2, this._hinten0],
        "21": [this._vorne2, this._hinten1],
        "22": [this._vorne2, this._hinten2],
        "23": [this._vorne2, this._hinten3],
        "24": [this._vorne2, this._hinten4],
        "25": [this._vorne2, this._hinten5],
        "26": [this._vorne2, this._hinten6],
        "27": [this._vorne2, this._hinten7],
        "28": [this._vorne2, this._hinten8],
        "29": [this._vorne2, this._hinten9],
        "30": [this._vorne3, this._hinten0],
        "31": [this._vorne3, this._hinten1],
        "32": [this._vorne3, this._hinten2],
        "33": [this._vorne3, this._hinten3],
        "34": [this._vorne3, this._hinten4],
        "35": [this._vorne3, this._hinten5],
        "36": [this._vorne3, this._hinten6],
        "37": [this._vorne3, this._hinten7],
        "38": [this._vorne3, this._hinten8],
        "39": [this._vorne3, this._hinten9],
        "40": [this._vorne4, this._hinten0],
        "41": [this._vorne4, this._hinten1],
        "42": [this._vorne4, this._hinten2],
        "43": [this._vorne4, this._hinten3],
        "44": [this._vorne4, this._hinten4],
        "45": [this._vorne4, this._hinten5],
        "46": [this._vorne4, this._hinten6],
        "47": [this._vorne4, this._hinten7],
        "48": [this._vorne4, this._hinten8],
        "49": [this._vorne4, this._hinten9],
        "50": [this._vorne5, this._hinten0],
        "51": [this._vorne5, this._hinten1],
        "52": [this._vorne5, this._hinten2],
        "53": [this._vorne5, this._hinten3],
        "54": [this._vorne5, this._hinten4],
        "55": [this._vorne5, this._hinten5],
        "56": [this._vorne5, this._hinten6],
        "57": [this._vorne5, this._hinten7],
        "58": [this._vorne5, this._hinten8],
        "59": [this._vorne5, this._hinten9]
    };

    _parseArrayOrObject(letters, styleClass, input) {
        if (typeof input !== 'undefined' && input !== null) {
            if (Array.isArray(input)) {
                input.forEach(item => {
                    this._parseObject(letters, styleClass, item);
                });
            } else {
                this._parseObject(letters, styleClass, input);
            }
        }
    }

    _parseObject(letters, styleClass, object) {
        if (typeof object !== 'undefined' && object !== null) {
            Object.keys(object).forEach(y => {
                const highlightLetters = object[y];
                highlightLetters.forEach(x => {
                    letters[y - 1][x - 1].addStyle(styleClass);
                });
            });
        }
    }

    _parseTimeDefinition(letters, styleClass, definition) {
        if (typeof definition !== 'undefined' && definition !== null) {
            Object.keys(definition).forEach(listString => {
                const array = listString.split(',');
                const highlightLetters = definition[listString];
                array.forEach(item => this._parseArrayOrObject(letters, styleClass + item, highlightLetters));
            });
        }
    }

    parse() {
        const letters = [];
        this._layout.letters.forEach(string => {
            const line = [];
            for (let c = 0; c < string.length; c++) {
                const character = new Letter(string[c]);
                line.push(character);
            }
            letters.push(line);
        });
        this._parseArrayOrObject(letters, 'on', this._layout.permanentlyOn);
        if (typeof this._layout.seconds !== 'undefined' && this._layout.seconds !== null) {
            this._parseTimeDefinition(letters, 'second', this._layout.seconds);
        } else {
            this._parseTimeDefinition(letters, 'second', this._seconds);
        }
        this._parseTimeDefinition(letters, 'minute', this._layout.minutes);
        this._parseTimeDefinition(letters, 'hour', this._layout.hours);
        return letters;
    };
}
